@@ -23,126 +23,131 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <libxfce4util/libxfce4util.h>
+
 #include "xkb-cairo.h"
 #include "xkb-util.h"
-#include "xfce4-xkb-plugin.h"
-
-#ifndef HAVE_LIBRSVG_2_36_2
-#include <librsvg/rsvg-cairo.h>
-#endif
 
 #define XKB_PREFERRED_FONT "Courier New, Courier 10 Pitch, Monospace Bold"
 
-#define xkb_cairo_arc_for_flag(cr, x, y, r, a1, a2) \
-    xx = layoutx + width - 12 + x; \
-    yy = layouty + height - 12 + y; \
-    cairo_device_to_user (cr, &xx, &yy); \
-    cairo_arc (cr, xx, yy, r, a1, a2);
-
-#define xkb_cairo_arc_for_label(cr, x, y, r, a1, a2) xx = x; \
-    yy = y; \
-    cairo_device_to_user (cr, &xx, &yy); \
-    cairo_arc (cr, xx, yy, r, a1, a2);
-
-#define xkb_cairo_move_to(cr, x, y) xx = x; \
-    yy = y; \
-    cairo_device_to_user (cr, &xx, &yy); \
-    cairo_move_to (cr, xx, yy);
-
-
 void
 xkb_cairo_draw_flag (cairo_t *cr,
-                     const gchar *group_name,
-                     gint panel_size,
+                     const GdkPixbuf *image,
                      gint actual_width,
                      gint actual_height,
-                     gint width,
-                     gint height,
                      gint variant_markers_count,
-                     gint textsize,
-                     GdkColor fgcolor)
+                     guint max_variant_markers_count,
+                     guint scale)
 {
-    gchar *filename;
-    RsvgHandle *handle;
-    RsvgDimensionData dim;
     double scalex, scaley;
-    double xx, yy;
-    gint i;
-    double layoutx, layouty;
+    gint i, width, height;
+    double layoutx, layouty, img_width, img_height;
+    double radius, diameter;
+    guint spacing;
 
     g_assert (cr != NULL);
+    g_assert (image != NULL);
 
-    if (!group_name)
-        return;
+    width = gdk_pixbuf_get_width (image);
+    height = gdk_pixbuf_get_height (image);
 
-    filename = xkb_util_get_flag_filename (group_name);
-    handle = rsvg_handle_new_from_file (filename, NULL);
-    g_free (filename);
+    scalex = (double) (actual_width - 4) / width;
+    scaley = (double) (actual_height - 4) / height;
 
-    if (!handle)
-    {
-        xkb_cairo_draw_label (cr, group_name,
-                panel_size,
-                actual_width, actual_height,
-                width, height,
-                variant_markers_count,
-                textsize,
-                fgcolor);
-        return;
-    }
+    scalex *= scale / 100.0;
+    scaley *= scale / 100.0;
 
-    rsvg_handle_get_dimensions (handle, &dim);
+    img_width  = width * scalex;
+    img_height = height * scaley;
 
-    scalex = (double) (width - 4) / dim.width;
-    scaley = (double) (height - 4) / dim.height;
+    DBG ("scale x/y: %.3f/%.3f, dim w/h: %d/%d, scaled w/h: %.1f/%.1f",
+         scalex, scaley, width, height, scalex*width, scaley*height);
 
-    layoutx = (actual_width - width) / 2 + 2;
-    layouty = (actual_height - height) / 2 + 2;
+    layoutx = (actual_width - img_width) / 2;
+    layouty = (actual_height - img_height) / 2;
     cairo_translate (cr, layoutx, layouty);
 
     cairo_save (cr);
 
     cairo_scale (cr, scalex, scaley);
-    rsvg_handle_render_cairo (handle, cr);
+    gdk_cairo_set_source_pixbuf (cr, image, 0, 0);
+    cairo_paint (cr);
 
     cairo_restore (cr);
 
+    DBG ("actual width/height: %d/%d; img w/h: %.1f/%.1f; markers: %d, max markers: %d",
+         actual_width, actual_height, img_width, img_height,
+         variant_markers_count, max_variant_markers_count);
+    DBG ("layout x/y: %.1f/%.1f", layoutx, layouty);
+
+    if (variant_markers_count > 0)
+    {
+        diameter = 5.0;
+        spacing = 1;
+
+        /* check if the flag is too small to draw variant markers inside it */
+        if ((diameter + spacing) * (max_variant_markers_count-1) > img_width - 2)
+        {
+            /* draw markers below the flag */
+            diameter = 4;
+            spacing  = 0;
+            layoutx  = actual_width / 2 + (max_variant_markers_count - 2) * diameter / 2;
+            layouty  = (actual_height + img_height) / 2 + diameter + 1;
+            DBG ("small flag");
+        }
+        else
+        {
+            /* draw markers inside the flag */
+            spacing  = 1;
+            layoutx += img_width  - diameter / 2 - 1;
+            layouty += img_height - diameter / 2 - 1;
+            DBG ("large flag");
+        }
+
+        radius = diameter / 2.0;
+
+        if (layouty > actual_height - radius)
+            layouty = actual_height - radius;
+        if (layoutx > actual_width - radius)
+            layoutx = actual_width - radius;
+    }
+
     /* draw variant_markers_count circles */
     for (i = 0; i < variant_markers_count; i++)
     {
+        gint x, y;
+
         cairo_set_source_rgb (cr, 0, 0, 0);
 
         cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
         cairo_set_line_width (cr, 1);
 
-        xkb_cairo_arc_for_flag (cr, -(7 * i) + 4, 4, 2.5, 0, 2 * G_PI);
+        x = layoutx - (diameter + spacing) * i + 0.5;
+        y = layouty;
+
+        DBG ("variant center x/y: %d/%d, diameter: %.1f, spacing: %d",
+             x, y, diameter, spacing);
+        cairo_arc (cr, x, y, radius, 0, 2 * G_PI);
 
         cairo_set_source_rgb (cr, 0, 0, 0);
         cairo_fill_preserve (cr);
         cairo_set_source_rgb (cr, 1, 1, 1);
         cairo_stroke (cr);
     }
-
-    rsvg_handle_close (handle, NULL);
-    g_object_unref (handle);
 }
 
 void
 xkb_cairo_draw_label (cairo_t *cr,
                       const gchar *group_name,
-                      const gint panel_size,
-                      const gint actual_width,
-                      const gint actual_height,
-                      const gint width,
-                      const gint height,
-                      const gint variant_markers_count,
-                      const gint textsize,
-                      const GdkColor fgcolor)
+                      gint actual_width,
+                      gint actual_height,
+                      gint variant_markers_count,
+                      guint scale,
+                      GdkRGBA rgba)
 {
     gchar *normalized_group_name;
     gint pango_width, pango_height;
     double layoutx, layouty, text_width, text_height;
-    double xx, yy;
     double scalex, scaley;
     gint i;
     double radius, diameter;
@@ -155,16 +160,12 @@ xkb_cairo_draw_label (cairo_t *cr,
     DBG ("actual width/height: %d/%d; markers: %d",
          actual_width, actual_height, variant_markers_count);
 
-    layout = pango_cairo_create_layout (cr);
-    normalized_group_name = xkb_util_normalize_group_name (group_name);
+    normalized_group_name = xkb_util_normalize_group_name (group_name, FALSE);
 
-    if (!normalized_group_name ||
-        !g_utf8_validate (normalized_group_name, -1, NULL))
-    {
-        g_object_unref (layout);
-        g_free (normalized_group_name);
+    if (!normalized_group_name)
         return;
-    }
+
+    layout = pango_cairo_create_layout (cr);
 
     pango_layout_set_text (layout, normalized_group_name, -1);
 
@@ -172,22 +173,11 @@ xkb_cairo_draw_label (cairo_t *cr,
     pango_layout_set_font_description (layout, desc);
     pango_font_description_free (desc);
 
-    gdk_cairo_set_source_color (cr, &fgcolor);
+    gdk_cairo_set_source_rgba (cr, &rgba);
     pango_layout_get_pixel_size (layout, &pango_width, &pango_height);
     DBG ("pango_width/height: %d/%d", pango_width, pango_height);
 
-    switch (textsize){
-        case DISPLAY_TEXTSIZE_SMALL:
-        default:    /* catch misconfiguration */
-            scalex = scaley = 0.475;
-            break;
-        case DISPLAY_TEXTSIZE_MEDIUM:
-            scalex = scaley = 0.7;
-            break;
-        case DISPLAY_TEXTSIZE_LARGE:
-            scalex = scaley = 1;
-            break;
-    }
+    scalex = scaley = scale / 100.0;
 
     DBG ("txt size scale x/y: %.2f/%.2f", scalex, scaley);
 
@@ -201,7 +191,7 @@ xkb_cairo_draw_label (cairo_t *cr,
     {
         text_width = actual_width - 3 - (variant_markers_count) * diameter;
     }
-    else if (textsize == DISPLAY_TEXTSIZE_LARGE)
+    else if (scale >= 99.5)
     {
         text_width -= 3;
     }
@@ -217,8 +207,8 @@ xkb_cairo_draw_label (cairo_t *cr,
     DBG ("layout x/y: %.2f/%.2f scale x/y: %.2f/%.2f, radius: %.2f",
          layoutx, layouty, scalex, scaley, radius);
 
-    xkb_cairo_move_to (cr, layoutx, layouty);
     cairo_save (cr);
+    cairo_move_to (cr, layoutx, layouty);
     cairo_scale (cr, scalex, scaley);
     pango_cairo_show_layout (cr, layout);
     cairo_restore (cr);
@@ -227,7 +217,7 @@ xkb_cairo_draw_label (cairo_t *cr,
     {
         cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
         cairo_set_line_width (cr, 1);
-        xkb_cairo_arc_for_label (cr,
+        cairo_arc (cr,
                 layoutx + text_width + 3 + (diameter * i),
                 layouty + text_height - (text_height / 5),
                 radius, 0, 2 * G_PI
@@ -239,3 +229,64 @@ xkb_cairo_draw_label (cairo_t *cr,
     g_object_unref (layout);
 }
 
+void
+xkb_cairo_draw_label_system (cairo_t *cr,
+                             const gchar *group_name,
+                             gint actual_width,
+                             gint actual_height,
+                             gint variant_markers_count,
+                             const PangoFontDescription *desc,
+                             GdkRGBA rgba)
+{
+    gchar *normalized_group_name;
+    gint pango_width, pango_height;
+    double layoutx, layouty;
+    gint i;
+    double radius, diameter;
+    PangoLayout *layout;
+
+    g_assert (cr != NULL);
+
+    DBG ("actual width/height: %d/%d; markers: %d",
+         actual_width, actual_height, variant_markers_count);
+
+    normalized_group_name = xkb_util_normalize_group_name (group_name, TRUE);
+
+    if (!normalized_group_name)
+        return;
+
+    layout = pango_cairo_create_layout (cr);
+
+    pango_layout_set_text (layout, normalized_group_name, -1);
+    pango_layout_set_font_description (layout, desc);
+
+    gdk_cairo_set_source_rgba (cr, &rgba);
+    pango_layout_get_pixel_size (layout, &pango_width, &pango_height);
+    DBG ("pango_width/height: %d/%d", pango_width, pango_height);
+
+    layoutx = (double) (actual_width - pango_width) / 2;
+    layouty = (actual_height - pango_height) / 2;
+    radius = pango_height / 10;
+    diameter = 2 * radius;
+
+    DBG ("layout x/y: %.2f/%.2f, radius: %.2f", layoutx, layouty, radius);
+
+    cairo_move_to (cr, layoutx, layouty);
+    pango_cairo_show_layout (cr, layout);
+
+    for (i = 0; i < variant_markers_count; i++)
+    {
+        cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
+        cairo_set_line_width (cr, 1);
+        cairo_arc (cr,
+                layoutx + (pango_width - (2 * variant_markers_count - 2) * diameter) / 2
+                    + 2 * diameter * i,
+                layouty + pango_height + radius,
+                radius, 0, 2 * G_PI
+        );
+        cairo_fill (cr);
+    }
+
+    g_free (normalized_group_name);
+    g_object_unref (layout);
+}
