@@ -26,34 +26,32 @@
 #ifndef _XKB_CAIRO_H_
 #define _XKB_CAIRO_H_
 
-#include <glib.h>
-#include <glib/gprintf.h>
 #include <gdk/gdk.h>
 #include <cairo/cairo.h>
-#include <librsvg/rsvg.h>
 #include <pango/pangocairo.h>
 
-void        xkb_cairo_draw_flag     (cairo_t *cr,
-                                     const gchar *flag_name,
-                                     gint panel_size,
-                                     gint actual_width,
-                                     gint actual_height,
-                                     gint width,
-                                     gint height,
-                                     gint variant_markers_count,
-                                     gint textsize,
-                                     GdkColor fgcolor);
+void        xkb_cairo_draw_flag         (cairo_t *cr,
+                                         const GdkPixbuf *image,
+                                         gint actual_width,
+                                         gint actual_height,
+                                         gint variant_markers_count,
+                                         guint max_variant_markers_count,
+                                         guint scale);
 
-void        xkb_cairo_draw_label    (cairo_t *cr,
-                                     const gchar *group_name,
-                                     const gint panel_size,
-                                     const gint actual_width,
-                                     const gint actual_height,
-                                     const gint width,
-                                     const gint height,
-                                     const gint variant_markers_count,
-                                     const gint textsize,
-                                     const GdkColor fgcolor);
+void        xkb_cairo_draw_label        (cairo_t *cr,
+                                         const gchar *group_name,
+                                         gint actual_width,
+                                         gint actual_height,
+                                         gint variant_markers_count,
+                                         guint scale,
+                                         GdkRGBA rgba);
+
+void        xkb_cairo_draw_label_system (cairo_t *cr,
+                                         const gchar *group_name,
+                                         gint actual_width,
+                                         gint actual_height,
+                                         gint variant_markers_count,
+                                         const PangoFontDescription *desc,
+                                         GdkRGBA rgba);
 
 #endif
-
