@@ -24,11 +24,8 @@
  */
 
 #include <string.h>
-#include <librsvg/rsvg.h>
-#include <cairo/cairo.h>
 
 #include "xkb-util.h"
-#include "xkb-config.h" // TODO : REMOVE THIS
 
 gchar*
 xkb_util_get_flag_filename (const gchar* group_name)
@@ -38,7 +35,13 @@ xkb_util_get_flag_filename (const gchar* group_name)
     if (!group_name)
         return NULL;
 
-    filename = g_strconcat (FLAGSDIR, "/", group_name, ".svg", NULL);
+    filename = g_strconcat (g_get_user_data_dir (), "/", FLAGSRELDIR, "/", group_name, ".svg", NULL);
+
+    if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+    {
+        g_free (filename);
+        filename = g_strconcat (DATADIR, "/", FLAGSRELDIR, "/", group_name, ".svg", NULL);
+    }
 
     return filename;
 }
@@ -64,7 +67,7 @@ xkb_util_get_layout_string (const gchar *group_name, const gchar *variant)
 }
 
 gchar*
-xkb_util_normalize_group_name (const gchar* group_name)
+xkb_util_normalize_group_name (const gchar* group_name, const gboolean capitalize)
 {
     const gchar *c;
     gchar *result;
@@ -74,22 +77,30 @@ xkb_util_normalize_group_name (const gchar* group_name)
     if (!group_name)
         return NULL;
 
-    if (strlen (group_name) <= 3)
-        return g_strdup (group_name);
+    cut_length = strlen (group_name);
 
-    for (c = group_name; *c; c++)
+    if (cut_length > 3)
     {
-        if (!((*c >= 'a' && *c <= 'z') || (*c >= 'A' && *c <= 'Z')))
+        for (c = group_name; *c; c++)
         {
-            index_of_na = group_name - c;
-            break;
+            if (!((*c >= 'a' && *c <= 'z') || (*c >= 'A' && *c <= 'Z')))
+            {
+                index_of_na = c - group_name;
+                break;
+            }
         }
+
+        cut_length = (index_of_na != -1 && index_of_na <= 3) ? index_of_na : 3;
     }
 
-    cut_length = (index_of_na != -1 && index_of_na <= 3) ? index_of_na : 3;
-
-    result = g_strndup (group_name, cut_length);
+    if (capitalize)
+    {
+        result = g_ascii_strup (group_name, cut_length);
+    }
+    else
+    {
+        result = g_strndup (group_name, cut_length);
+    }
 
     return result;
 }
-
